//pomocne funkce pouzivane pri ruznych prilezitostech
const POST_BOUNDARY='#az09za90#';

//------------------------------------------------------------------------------
//deklarace funkci definovanych v jinych souborech
procedure cbStateOnChange(sender:TObject); forward;   //LabForGg.fceForm.pas
procedure cbCountryOnChange(sender:TObject); forward; //LabForGg.fceForm.pas
//------------------------------------------------------------------------------

function GetMyDir:string;
begin
  Result:=RegExSubstitute('(.*\\)(.*)$',GEOGET_SCRIPTFULLNAME,'$1');
end;

function GetGpxDir:string;
begin
  Result:=GetMyDir()+'\GpxDownload\';
end;

function GetOfflineDir:string;
begin
  Result:=GetMyDir()+'Offline\';
  ForceDirectories(Result);
end;

function GetIniPathname:string;
begin
  Result:=ReplaceString(GEOGET_SCRIPTFULLNAME,'.ggp.pas','.ini');
end;

function GetServerSeriesPathname:string;
begin
  Result:=GetMyDir()+'ListOfSeries-server.txt';
end;

function GetApiRadiusParsePathname:string;
begin
  Result:=GetMyDir()+'ApiRadiusParse.txt';
end;

procedure Log(sMsg:string);  //zaznam zpravy do protokolu
var str:string;
begin
{$ifdef DEBUG_HELPER} LDH(sMsg); {$endif}
  if(fProt<>nil) then begin
    str:=sMsg+CRLF;
    fProt.Write(str,Length(str));
  end;
end;

procedure ProtStart; //vytvori soubor s protokolem
begin
  fProt:=TFileStream.Create(sProtFile,fmCreate or fmShareDenyNone);
  Log(FormatDateTime('hh:nn:ss.zzz',Now())+'  Start');
end;

procedure ProtEnd;
var s:string;
    aList:TStringList;
    i:integer;
begin
  Log(FormatDateTime('hh:nn:ss.zzz',Now())+'  End');
  fProt.Free();
  fProt:=nil;
  //vypiseme vsechny radky s ERR: nebo WARN:
  s:='';
  aList:=TStringList.Create();
  try
    aList.Text:=FileToString(sProtFile);
    for i:=0 to aList.Count-1 do begin
      if(Pos('ERR:',aList[i])>0) then s:=s+aList[i]+CRLF;
      if(Pos('WARN:',aList[i])>0) then s:=s+aList[i]+CRLF;
    end;
  finally
    aList.Free()
  end;
  if(s<>'') then
    ShowLongMessage(('Problmy pi importu sri'),
      s+CRLF+CRLF+'Podrobn informace jsou v souboru '+sProtFile);
end;

function IsEvent(g:TGeo):boolean;
begin
  Result:=(g.CacheType='Cache In Trash Out Event') OR (g.CacheType='Event Cache')
      OR (g.CacheType='GPS Adventures Exhibit')  OR (g.CacheType='Community Celebration Event')
      OR (g.CacheType='Mega-Event Cache') OR (g.CacheType='Giga-Event Cache');
end;

function Labnum2Int(str:string):integer;
//prevede znakove cislo s pismeny na integer
var i,j:integer;
begin
  Result:=0;
  str:=UpperCase(str);
  for i:=0 to Length(str)-1 do begin
    j:=Pos(str[i+1],CISLOVANI_SADA)-1;
    if(j<0) then begin
      ShowMessage('ERR: Chybny znak v poradoven cisle LAB kese: '+str);
      exit;
    end;
    Result:=Result*(Length(CISLOVANI_SADA))+j;
  end;
end;

function Int2Labnum(iVal:integer):string;
//prevede integer na znakove cislo s pismeny
var i:integer;
begin
  Result:='';
  while(iVal>0) do begin
    i:=(iVal mod Length(CISLOVANI_SADA));
    Result:=CISLOVANI_SADA[i+1]+Result;
    iVal:=iVal/Length(CISLOVANI_SADA);
  end;
  if(Result='') then Result:='0';
end;

function SerieNameFromName(sName:string):string;
  //z formatu <jmeno serie>/<jmeno labky>[ by <autor>] vrati jen jmeno serie
begin
  Result:=Fetch(sName,'/');
end;

function LabNameFromName(sName:string):string;
  //z formatu <jmeno serie>/<jmeno labky>[ by <autor>] vrati jen jmeno labky
begin
  Result:='';
  Fetch(sName,'/');
  Result:=RegexSubstitute('(.*)\s+by\ ',sName,'$1');
  if(Result='') then Result:=sName;
end;

function IsAscii(str:string):boolean;
var i:integer;
begin
  Result:=false;
  for i:=1 to Length(str) do begin
    if(str[i]<' ') or (str[i]>'~') then exit; //mimo ASCII
  end;
  Result:=true;
end;

function AddPassword(sUrl:string):string;
begin
  Result:=sUrl;
  if(sWEBPUB_USER<>'') and (sWEBPUB_PWD<>'') then begin
    if(Pos('http',sUrl)>=1) then begin
      Result:=ReplaceString(Result,'http://','http://'+sWEBPUB_USER+':'+sWEBPUB_PWD+'@');
      Result:=ReplaceString(Result,'https://','https://'+sWEBPUB_USER+':'+sWEBPUB_PWD+'@');
    end
    else Result:=sWEBPUB_USER+':'+sWEBPUB_PWD+'@'+sUrl;
  end;
end;

function PostBoundary():string;
begin
  Result:='--'+POST_BOUNDARY;
end;

function AddPostParam(sParams,sParName,sValue:string):string;
begin
  Result:=sParams+PostBoundary()+CRLF
    +'Content-Disposition: form-data; name="'+sParName+'"'+CRLF+CRLF+sValue+CRLF;
end;

function AddPostSender(sParams:string):string;
begin
  Result:=AddPostParam(sParams,'sender','plugin');
end;

function GcFromLabId(sLabId:string):string;
var sSql:string;
    aList:TStringList;
begin
  Result:='';
  aList:=TStringList.Create();
  try
    GEOGET_DB.ParamsClear();
    GEOGET_DB.AddParamText(':Cat','LABid');
    GEOGET_DB.AddParamText(':IdLab',sLabId);
    sSql:='SELECT gt.id FROM geotag gt, geotagcategory gtc, geotagvalue gtv '
      +'WHERE gtc.key=gt.ptrkat AND gtc.value=:Cat COLLATE NOCASE AND gtv.key=gt.ptrvalue '
      +'AND gtv.value=:IdLab COLLATE NOCASE ORDER BY gt.id';
    GEOGET_DB.GetTableStrings(sSql,aList);
//{$ifdef DEBUG_HELPER} LDH('aList='+aList.Text); {$endif}
    Result:=Trim(ReplaceString(Trim(aList.Text),CRLF,','));
  finally
    aList.Free();
  end;
end;

function IsLabIdInGg(sLabId:string):boolean;
begin
  Result:=(GcFromLabId(sLabId)<>'');
end;

function IsLabIdOnServer(sLabId:string):boolean;
var sRes,sUrl,sText:string;
begin
  //GgForm_eIdTag.Text:='df641e5c-f740-4cec-b2c7-c25b7d9ff149x'; //test, moje labka
  //sestavime parametry
  sText:=AddPostParam('','f','getNameForId');
  sText:=AddPostParam(sText,'fn',sLabId);
  sText:=AddPostSender(sText)+PostBoundary();  //odesilatel, podle nej server pozna, ze jde o plugin a ne o formular
  sUrl:=AddPassword(sWEBPUB_URL); //uzivatel a heslo
  {$ifdef DEBUG_HELPER} LDH('### Test labek na serveru: URL='+sUrl);{$endif}
  HttpAsk('POST',sUrl,'multipart/form-data; boundary='+POST_BOUNDARY,sText,sRes);
  //StringToFile(sRes,GetMyDir()+'HttpResponse.txt');
  Result:=(sRes<>'ERR: Not found'); //serie neexistuje, vse je OK
end;

function IsNickAdmin:boolean; //zkontroluje, zda uzivatel GG je adminem pro spravcovani nektereho z kraju
var sRes,sUrl,sText,sAdmin:string;
begin
  Result:=false;
  sText:=AddPostParam('','f','checkAdmin');
  //pokud uzivatel GG neba stejne prihlasovaci jmeno na serveru, nastavime jej
  sAdmin:=GgForm_eGpxAuthor.Text;
  sText:=AddPostParam(sText,'fn',sAdmin);
  sText:=AddPostSender(sText)+PostBoundary();  //odesilatel, podle nej server pozna, ze jde o plugin a ne o formular
  sUrl:=AddPassword(sWEBPUB_URL); //uzivatel a heslo
  //{$ifdef DEBUG_HELPER} LDH('### Opravnenost Admina (spravcovani kraju): URL='+sUrl);{$endif}
  HttpAsk('POST',sUrl,'multipart/form-data; boundary='+POST_BOUNDARY,sText,sRes);
  {$ifdef DEBUG_HELPER} LDH('### Uzivatel GG: '+sAdmin+', opravneny spravce? '+sRes);{$endif}
  if(sRes='yes') then Result:=true;
end;

function ExportHeader: string;
begin
  Result:='<?xml version="1.0" encoding="utf-8" standalone="no"?>'+CRLF;
  Result:=Result+'<gpx xmlns="http://www.topografix.com/GPX/1/1"'
    +' creator="'+Geoget_Version
      +', plugin LabForGg '+GetCurrentPluginVersion()+', '+GgForm_eGpxAuthor.Text
    +'"'
    +' version="1.1"'
    +' xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"'
    +' xmlns:gpxx="http://www.garmin.com/xmlschemas/GpxExtensions/v3"'
    +' xsi:schemaLocation="'
    +'https://www.geoget.cz/GpxExtensions/v2 https://www.geoget.cz/GpxExtensions/v2/geoget.xsd'
    +' http://www.groundspeak.com/cache/1/0 http://www.groundspeak.com/cache/1/0/cache.xsd'
    +' http://www.topografix.com/GPX/1/1 http://www.topografix.com/GPX/1/1/gpx.xsd'
    +' http://www.garmin.com/xmlschemas/GpxExtensions/v3 http://www8.garmin.com/xmlschemas/GpxExtensionsv3.xsd'
    +'">'+CRLF
    +' <metadata>'+CRLF
    +'   <desc>Geoget data file</desc>'+CRLF
    +'   <time>'+formatdatetime('yyyy"-"mm"-"dd"T"hh":"nn":"ss"."zzz', now)+'</time>'+CRLF
    +' </metadata>'+CRLF
    +CRLF;
end;

function MakeSafeString(str:string):string;
begin
  Result:=UtfToAscii(AnsiToUtf((str)));
  //vyhazeme nebezpecne znaky, \ musi byt samostatne, to nejak tom negativnim vyberu nefunguje
  //znak '-' musi byt povolen kvuli LabId
  Result:=RegexReplace('([^a-zA-z0-9 _-]|[/\\\[\]])',Result,'',false);
  Result:=ReplaceString(Result,' ','_');
end;

function MakeFilename(sLabName:string):string;
begin
  Result:='LAB_'+MakeSafeString(sLabName)+'.gpx';
end;

function GetLabFileName:string;
begin
  Result:=SAVE_DIR;
  if(Result='') then Result:=GEOGET_DATADIR+'\';
  Result:=ReplaceString(Result+'\','\\','\'); //pridame na konec lomitko, pokud je to potreba
  Result:=Result+MakeFilename(GgForm_eName.Text);
end;

procedure FillProfiles;   //naplni seznam profilu z INI souboru
var ini:TMemIniFile;
    aSec:TStringList;
    i:integer;
begin
  GgForm_cbProfile.Items.Text:=''; //vymazeme seznam
  //nacteme seznam vsech sekci v ini souboru a vyhodime sekce, ktere odpovidaji
  //jmenu nektere z databazi, resp jen takove sekce, ktere maji zadanou polozku
  //Bonus
  ini:=TMemIniFile.Create(GetIniPathname());
  try
    aSec:=TStringList.Create();
    try
      ini.ReadSections(aSec);
      for i:=0 to aSec.Count-1 do begin
        //ignorujeme sekce, ktere patri databazim (datum aktualizace)
        if(ini.ReadInteger(aSec[i],'Bonus',-1) >-1) then begin
          GgForm_cbProfile.Items.Add(aSec[i]);
        end;
      end;
    finally
      aSec.Free();
    end;
  finally
    if(ini<>nil) then ini.Free();
  end;
  GgForm_cbProfile.ItemIndex:=0;
end;

procedure FillState;
var s,s2,s4,sCountry,sRow:string;
begin
  //naplnime seznam kraju pro aktualne vybrany stat
  GgForm_cbState.Items.Text:='';
  if(GgForm_cbCountry.ItemIndex<0) then exit;
  sCountry:=GgForm_cbCountry.Items[GgForm_cbCountry.ItemIndex];
  s:=UtfToAnsi(FileToString(GEOGET_DATADIR+'\countryid.txt'));
  while(s<>'') do begin
    sRow:=Fetch(s,CRLF);
    Fetch(sRow,'|'); //zahodime ID statu
    s2:=Fetch(sRow,'|'); //stat
    if(sCountry<>s2) then continue;
    s4:=Fetch(sRow,'|');
    if(s4='-1') then continue;
    s4:=Fetch(sRow,'|');
    GgForm_cbState.Items.Add(s4);
  end;
  cbStateOnChange(nil); //nastavime spravnou barvu pozadi jako priznak povinne hodnoty
end;

procedure FillCountry;
var s,s2,sCountry,sRow:string;
begin
  GgForm_cbCountry.Items.Text:='';
  s:=UtfToAnsi(FileToString(GEOGET_DATADIR+'\countryid.txt'));
  while(s<>'') do begin
    sRow:=Fetch(s,CRLF);
    Fetch(sRow,'|'); //zahodime ID statu
    s2:=Fetch(sRow,'|');
    if(sCountry<>s2) then begin
      sCountry:=s2;
      GgForm_cbCountry.Items.Add(s2);
    end;
  end;
  GgForm_cbCountry.ItemIndex:=-1;
  FillState();  //vyplnime kraje pro zvoleny stat
end;

procedure SetCbIdEditable(bEdit:boolean);
begin
  if(bEdit) then GgForm_cbId.Style:=csDropDown
  else GgForm_cbId.Style:=csDropDownList;
end;

procedure ClearTabGpx;
//var dt:TDateTime;
begin
  //pokud je nastaven priznak nove serie, formular se vymazat nesmi
  if(GgForm_cbId.Style=csDropDown) then exit; //s moznosti napsat GC kod, nemazeme formular
  //nemazat, aby bylo mozne seznam vyuzivat i dal
  {
    ClearCbId(); //vymazeme i jindy ponechavany seznam
  }
  GgForm_eName.Text:='';
  GgForm_eNameApi.Text:='';
  GgForm_eNick.Text:='';
  GgForm_eOwnerId.Text:='';
  GgForm_bBonus.Checked:=false;
  GgForm_bSeq.Checked:=false;
  GgForm_bEvent.Checked:=false;
  GgForm_bArchiv.Checked:=false;
  GgForm_cbCountry.ItemIndex:=-1;
  GgForm_cbState.ItemIndex:=-1;
  GgForm_mText.Lines.Text:='';
  GgForm_eIdTag.Text:='';
  GgForm_eUrlQr.Text:='';
  GgForm_eCoord.Text:='';
  GgForm_cbLabView.ItemIndex:=-1;
  GgForm_eLabName.Text:='';
  GgForm_eLabCoord.Text:='';
  GgForm_eLabIdOrd.Text:='';
  GgForm_mLabText.Lines.Text:='';
  GgForm_cbCount.ItemIndex:=4;
  GgForm_cbNumbersOnly.ItemIndex:=1;
  GgForm_cbNumbersOnly.Enabled:=false; //zakazeme, protoze server predpoklada cislovani s pismeny a jen s cislama by mohl byt chybne pocet labek v serii
  GgForm_butNameApiCopyRed.Visible:=false;
  GgForm_butNameApiCopy.Visible:=true;
  GgForm_butNameApiCopy.Enabled:=false;
  GgForm_butNameApiCopyRed.Hint:='';
  GgForm_butNameApiCopy.Hint:='';
  GgForm_dtCreate.Date:=Trunc(now());
  GgForm_eChangeGpx.Text:='';
  GgForm_bGGView.Checked:=false;
  GgForm_butLabOrder.Enabled:=false;
  GgForm_butLabsSave.Enabled:=false;
end;

procedure AddGgInfo(sId:string); //do formulare doplni hodnoty, ktere uz serie v databazi ma
var s,s1,sTags:string;
    g:TGeo;
begin
  s1:=GcFromLabId(sId); //labky v serii
  if(s1='') then exit;
  s:=Fetch(s1,',');     //pouzijeme jen prvni z nich
  s1:=s;
  GgForm_cbId.Text:=Fetch(s1,'-');  //prepisem kod rodicovske kese
  g:=TGeo.Create();
  try
    g.LoadByGc(s);
    //pokud LABka existuje, prevezmeme z ni hodnoty
    if(not g.IsEmptyCoord()) then begin
      GgForm_cbCountry.ItemIndex:=GgForm_cbCountry.Items.IndexOf(UtfToAnsi(g.Country));
      cbCountryOnChange(nil);
      GgForm_cbState.ItemIndex:=GgForm_cbState.Items.IndexOf(UtfToAnsi(g.State));
      cbStateOnChange(nil);
      GgForm_dtCreate.Date:=g.Hidden;
      //a ted nastaveni podle tagu
      //od 24.11.2020 toto neprenasime, vzdy zaciname s prazdnym
      //GgForm_eChangeGpx.Text:=g.TagValues('LABchange');
      sTags:=g.TagValues('LABflags');
      GgForm_bSeq.Checked:=(Pos('IsSequential',sTags)>0);
      GgForm_bBonus.Checked:=(Pos('HaveBonus',sTags)>0);
      GgForm_bEvent.Checked:=(Pos('ForEvent',sTags)>0);
      GgForm_bArchiv.Checked:=g.IsArchived();
      //hodnoty z databaze zobrazujeme, jen kdyz zadne zobrazene nejsou
      if(GgForm_eIdTag.Text='') then GgForm_eIdTag.Text:=g.TagValues('LABid');
      if(GgForm_eCoord.Text='') then GgForm_eCoord.Text:={CoordDD2Str}(g.TagValues('LABcoord'));
      if(GgForm_eUrlQr.Text='') then begin
        //GgForm_eUrlQr.Text:=UtfToAnsi(g.TagValues('LABlink'));
        //zkusime nacist ze short_description 1. odkaz, pokud je tam .goto. pouzijeme ho,
        // je to FirebaseDynamicLink
        s:=RegExSubstitute('<a\s+href=\"(.*?)\s*\"',g.ShortDescription,'$1');
        //{$ifdef DEBUG_HELPER} LDH('XXXXXXXXXXXXXXXXXXX Link='+s+CRLF+g.ShortDescription); {$endif}
        //pokud je tam opravdu FirebaseDynamicLink, pouzijeme to
        if(Pos('.page.',s)>0) then GgForm_eUrlQr.Text:=s;
      end;
      //pokud tam stale nic neni, zkusime SmartLink
      if(GgForm_eUrlQr.Text='') then GgForm_eUrlQr.Text:=UtfToAnsi(g.TagValues('LABsmartLink'));
    end;
  finally
    g.Free();
  end;
end;

//zkontroluje, zda existuje index podle GUID, pokud ne, vytvori jej,
//aby bylo hledani podle GUID (ID konkretni labky) co nejrychlejsi
const GUID_IDX_NAME='LabForGg_Guid_idx';
procedure CheckAndCreateGuidIndex;
var sSql,s:string;
begin
  {$ifdef DEBUG_HELPER} LDH('$$$$$ Kontrola existence indexu '+GUID_IDX_NAME); {$endif}
  sSql:='SELECT SQL FROM sqlite_master WHERE type="index" '
    +'AND tbl_name = "geocache" AND name = "'+GUID_IDX_NAME+'"';
  s:=Geoget_DB.GetTableString(sSql);
  if(s='') then begin
    {$ifdef DEBUG_HELPER} LDH('   ... neexistuje, vytvorim ho'); {$endif}
    Geoget_DB.ExecSQL('CREATE INDEX LabForGg_Guid_idx ON geocache(id)');
  end
    {$ifdef DEBUG_HELPER}else begin LDH('   ... jiz existuje, vytvoreni preskoceno'); end {$endif}
  ;
end;

function ParentCacheTest:boolean;
var s:string;
    i:integer;
begin
  Result:=false;
  while(Result=false) do begin
    GEOGET_DB.AddParamText(':id',GgForm_cbId.Text);
    s:=GEOGET_DB.GetTableString('SELECT id FROM geocache WHERE id=:id');
    Result:=(s=GgForm_cbId.Text);
    if(Result=false) then begin
      i:=MessBox('Rodiovsk ke '+GgForm_cbId.Text+' nen v databzi.'+CRLF+CRLF+'Importovat ji z ceocaching.com?',
        'Chyba',4);
      if(i=7) then break; //nema byt importovana, vracime se s chybou
      //zkusime ji importovat
      GgForm.Hide();
      GcLiveUpdateListing2(GgForm_cbId.Text,1); //aktualizujeme v LITE rezimu
      GgForm.Show();
      //a opakovanim cyklu zkusime, zda se aktualizace podarila
    end;
  end;
end;

//nacte pres API jmeno rodicovske kese a jejiho autora
procedure GetParentNameAndAuthorFromApi(var name, author:string);
var
  sCmd,sJson:string;
  iErr:integer;
  rJson:TSimpleJSon;
begin
  name:='';
  author:='';
  GeoBusyCaption('Zskn jmna rodiovsk kee a autora (pes API)');
  //GC kod rodicovske kese: GgForm_cbId.Text
  sCmd:='v1/geocaches/'+GgForm_cbId.Text+
    +'?lite=true'
    +'&fields=referenceCode,name,ownerCode,ownerAlias,terrain,difficulty';
  iErr:=GcLiveRest('GET',sCmd,'','',sJson);
  {$ifdef DEBUG_HLEPER} LDH('navrat z API: iErr='+intToStr(iErr)); {$endif}
  {$ifdef DEBUG_HLEPER} LDH('JSON: '+sJson); {$endif}
  //StringToFile(sJson,GetMyDir()+'GcInfo.json');
  rJson:=TSimpleJson.Create();
  try
    if(rJson.ParseString(sJson)) then begin
      name:=rJson.asString('name');
      author:=rJson.asString('ownerAlias');
    end;
  finally
    rJson.Free();
  end;
end;

procedure SaveTimeToIni(sVar:string); // ulozi cas importu do INI souboru pro pristi pouziti
var s:string;
    ini:TMemIniFile;
begin
  ini:=TMemIniFile.Create(GetIniPathname());
  try
    s:=FormatDatetime('dd.mm.yyyy hh:nn:ss',now());
    ini.WriteString(GEOGET_DBNAME,sVar,s);
    ini.UpdateFile();          //promitneme zmeny na disk
  finally
    if(ini<>nil) then ini.Free();
  end;
end;

function GetTimeFromIni(sVar:string):TDateTime; // nacte cas z ini souboru
var s:string;
    ini:TMemIniFile;
    dtd, dtt:TDateTime;
    aYear, aMonth, aDay: word;
    aHour, aMinute, aSecond: word;
begin
  ini:=TMemIniFile.Create(GetIniPathname());
  try
    s:=ini.ReadString(GEOGET_DBNAME,sVar{'LastImport'},'');
    if(s<>'') then begin
      aDay := StrToIntDef(Copy(s, 1, 2), 0);
      aMonth := StrToIntDef(Copy(s, 4, 2), 0);
      aYear := StrToIntDef(Copy(s, 7, 4), 0);
      aHour := StrToIntDef(Copy(s, 12, 2), 0);
      aMinute := StrToIntDef(Copy(s, 15, 2), 0);
      aSecond := StrToIntDef(Copy(s, 18, 4), 0);
      dtd:=0;
      dtt:=0;
      if TryEncodeDate(aYear, aMonth, aDay, dtd) and
         TryEncodeTime(aHour, aMinute, aSecond, 0, dtt) then begin
        if(sVar='LastImport') then begin
          GgForm_dtSearchDate.Date := dtd;
          GgForm_dtSearchTime.Time:=dtt;
        end;
        Result:=dtd+dtt;
      end;
    end;
  finally
    if(ini<>nil) then ini.Free();
  end;
end;

procedure MakeDefaultIni;
begin
  StringToFile(
    '[Aktualizovat R]'+CRLF+'Id='+CRLF+'Name='+CRLF+'Author='+CRLF+'Country=Czechia'+CRLF
    +'State='+CRLF+'Bonus=2'+CRLF+'Seq=2'+CRLF+'Event=2'+CRLF+'Archiv=2'+CRLF
    +'PubGpx=1'+CRLF+'New=0'+CRLF+'Import=1'+CRLF+'SaveTime=1'+CRLF+CRLF
    +'[Ve]'+CRLF+'Id='+CRLF+'Name='+CRLF+'Author='+CRLF+'Country='+CRLF
    +'State='+CRLF+'Bonus=2'+CRLF+'Seq=2'+CRLF+'Event=2'+CRLF+'Archiv=2'+CRLF
    +'PubGpx=2'+CRLF+'New=0'+CRLF+'Import=0'+CRLF+'SaveTime=1'+CRLF
    +'UseDT=0'+CRLF+'AutorGpx='+CRLF
    ,GetIniPathName());
end;

procedure ProfileRead(sSec:string);
var ini:TMemIniFile;
begin
  ini:=TMemIniFile.Create(GetIniPathName());
  try
    GgForm_eSearchId.Text:=ini.ReadString(sSec,'Id','');
    GgForm_eSearchName.Text:=ini.ReadString(sSec,'Name','');
    GgForm_eSearchAuthor.Text:=ini.ReadString(sSec,'Author','');
    GgForm_eSearchCountry.Text:=ini.ReadString(sSec,'Country','');
    GgForm_eSearchState.Text:=ini.ReadString(sSec,'State','');
    GgForm_rSearchBonus.ItemIndex:=ini.ReadInteger(sSec,'Bonus',2);
    GgForm_rSearchSeq.ItemIndex:=ini.ReadInteger(sSec,'Seq',2);
    GgForm_rSearchEvent.ItemIndex:=ini.ReadInteger(sSec,'Event',2);
    GgForm_rSearchArchiv.ItemIndex:=ini.ReadInteger(sSec,'Archiv',2);
    GgForm_rSearchPub.ItemIndex:=ini.ReadInteger(sSec,'PubGpx',2);
    GgForm_bSearchMissing.Checked:=ini.ReadBool(sSec,'New',false);
    GgForm_bSearchImport.Checked:=ini.ReadBool(sSec,'Import',false);
    GgForm_bSearchSaveTime.Checked:=ini.ReadBool(sSec,'SaveTime',true);
    GgForm_bDT.Checked:=ini.ReadBool(sSec,'UseDT',false);
    GgForm_eSearchAutorGpx.Text:=ini.ReadString(sSec,'AutorGpx','');
  finally
    ini.Free();
  end;
end;

procedure ProfileSave(sSec:string);
var ini:TMemIniFile;
begin
  ini:=TMemIniFile.Create(GetIniPathName());
  try
    ini.WriteString(sSec,'Id',GgForm_eSearchId.Text);
    ini.WriteString(sSec,'Name',GgForm_eSearchName.Text);
    ini.WriteString(sSec,'Author',GgForm_eSearchAuthor.Text);
    ini.WriteString(sSec,'Country',GgForm_eSearchCountry.Text);
    ini.WriteString(sSec,'State',GgForm_eSearchState.Text);
    ini.WriteInteger(sSec,'Bonus',GgForm_rSearchBonus.ItemIndex);
    ini.WriteInteger(sSec,'Seq',GgForm_rSearchSeq.ItemIndex);
    ini.WriteInteger(sSec,'Event',GgForm_rSearchEvent.ItemIndex);
    ini.WriteInteger(sSec,'Archiv',GgForm_rSearchArchiv.ItemIndex);
    ini.WriteInteger(sSec,'PubGpx',GgForm_rSearchPub.ItemIndex);
    ini.WriteBool(sSec,'New',GgForm_bSearchMissing.Checked);
    ini.WriteBool(sSec,'Import',GgForm_bSearchImport.Checked);
    ini.WriteBool(sSec,'SaveTime',GgForm_bSearchSaveTime.Checked);
    ini.WriteBool(sSec,'UseDT',GgForm_bDT.Checked);
    ini.WriteString(sSec,'AutorGpx',GgForm_eSearchAutorGpx.Text);
    ini.UpdateFile();          //promitneme zmeny na disk
  finally
    ini.Free();
  end;
end;

function AgeOfFile(sFile:string):integer; //vraci stari souboru v minutach
var dt:TDateTime;
begin
  Result:=99999;
  if(FileExists(sFile)) then begin
    dt:=GetFileTime(sFile);
    Result:=Round((Now()-dt)/60); //stari v minutach
  {$ifdef DEBUG_HELPER} LDH('Stari souboru '+sFile+': '+IntToStr(Result));{$endif}
  end;
end;

function LabIdFromGc(sGc:string):string;
var sSql:string;
begin
  GEOGET_DB.AddParamText(':id',sGc);
  sSql:='SELECT gtv.value LabId_1 FROM'+CRLF
    +'geocache geotag gt, geotagcategory gtc, geotagvalue gtv'+CRLF
    +'WHERE gt.id=:id AND gtc.key=gt.ptrkat AND gtc.value="LABid" AND gtv.key=gt.ptrvalue';
  Result:=GEOGET_DB.GetTableString(sSql);
{$ifdef DEBUG_HELPER} LDH('GC='+sGc+' ... LabId='+Result); {$endif}
end;

